///////////////////////////////////////////////////////////////////////////////////////
//   
// Net radio tuner system
//   
//   Radio tuner consists of ONKYO T-422M, Pi Zero, PIC18F, LAN board and DAC board. 
//   This software controls PIC18F.
//
//   Briefing document(in Japanese) is available on https://www.snowinnovember.info
//
///////////////////////////////////////////////////////////////////////////////////////

/**
  Generated Main Source File

  Company:
    Microchip Technology Inc.

  File Name:
    main.c

  Summary:
    This is the main file generated using PIC10 / PIC12 / PIC16 / PIC18 MCUs

  Description:
    This header file provides implementations for driver APIs for all modules selected in the GUI.
    Generation Information :
        Product Revision  :  PIC10 / PIC12 / PIC16 / PIC18 MCUs - 1.78
        Device            :  PIC18F26K22
        Driver Version    :  2.00
*/

/*
    (c) 2018 Microchip Technology Inc. and its subsidiaries. 
    
    Subject to your compliance with these terms, you may use Microchip software and any 
    derivatives exclusively with Microchip products. It is your responsibility to comply with third party 
    license terms applicable to your use of third party software (including open source software) that 
    may accompany Microchip software.
    
    THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER 
    EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY 
    IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS 
    FOR A PARTICULAR PURPOSE.
    
    IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
    INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
    WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP 
    HAS BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO 
    THE FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL 
    CLAIMS IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT 
    OF FEES, IF ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS 
    SOFTWARE.
*/

#include "mcc_generated_files/mcc.h"

#include "lm7001_t-422m.h"
#include "command.h"

/*
                         Main application
 */

#define USE_ASM
//#define DISABLE_TIME_OUT

#define SPI_CMD_BYTES 3

//290us, Power on :458us, off:122us
#define FIP_PULSE_WIDTH_TH 0x4880

//Time out (ms)
#define SHUTDOWN_MAX_WAIT_TIME              20000
#define FORCE_SHUTDOWN_MAX_WAIT_TIME        20000
#define PI_STATUS_MAX_WAIT_TIME             5000
#define PI_STATUS_MAX_WAIT_TIME_FOR_INIT    60000

//Time out (sec)
#define NO_AUDIO_MAX_TIME                   15

//Time interval (ms)
#define POWER_OFF_TO_ON_INTERVAL            500
#define SHUTDOWN_COMP_TO_POWER_OFF_INTERVAL 500


#ifdef DISABLE_TIME_OUT
char  disable_time_out = 1; 
#else
char  disable_time_out = 0; 
#endif

unsigned char spi_buf[SPI_CMD_BYTES]; //3-stage buffer for 24bits
unsigned char spi_buf_cnt;//Number of valid data in buffer
unsigned char spi_buf_wp;
unsigned char spi_buf_rp;
char          pll_rx_en;

typedef struct {
  unsigned int   d;
  char  s;
  char  valid; 
} LM7001_pll;

LM7001_pll pll;

char          pll_rx_data_valid = 0;// pll_rx_data is valid

unsigned char fm_valid[NUM_OF_BYTES_FOR_FM]; //1 bit for one station
unsigned char am_valid[NUM_OF_BYTES_FOR_AM];

unsigned char fm_valid_valid;
unsigned char am_valid_valid;
char          sent_pll_valid;

char no_audio;

char tmr1_cnt;

#ifdef USE_ASM
char v_asm_h0;
char v_asm_l0;
char v_asm_h1;
char v_asm_l1;
#endif

void pi_power_off();
char pi_power_on(); 
char pi_shutdown(); 
void pi_force_shutdown();
char pi_operation(); 

void enable_audio_check();
void disable_audio_check();
unsigned char receive_valid(char *, int);

void spi_pll_rx_handler();
void int2_pll_rx_en_handler();
void timer1_int_handler();

void main(void) {
  int i;
  unsigned int fip_pulse_width;
  char power_sw_on;
  
  typedef enum {
    power_on,
    operation,
    shutdown,
    force_shutdown,
    power_off,
  } Pi_state;  
  
  Pi_state pi_state;
   
  // Initialize the device
  SYSTEM_Initialize();
  INT2_SetInterruptHandler(int2_pll_rx_en_handler);
  SPI1_SetIsrHandler(spi_pll_rx_handler);
  TMR1_SetInterruptHandler(timer1_int_handler);

    // If using interrupts in PIC18 High/Low Priority Mode you need to enable the Global High and Low Interrupts
    // If using interrupts in PIC Mid-Range Compatibility Mode you need to enable the Global and Peripheral Interrupts
    // Use the following macros to:

    // Enable the Global Interrupts
    //INTERRUPT_GlobalInterruptEnable();

    // Disable the Global Interrupts
    //INTERRUPT_GlobalInterruptDisable();

    // Enable the Peripheral Interrupts
    //INTERRUPT_PeripheralInterruptEnable();

    // Disable the Peripheral Interrupts
    //INTERRUPT_PeripheralInterruptDisable();
  
// Add your application code
  spi_buf_wp = 0;
  
  SSP1CON1bits.SSPEN = 1;
  INTCONbits.RBIE = 0;   //Only RBIF is used, but interrupt is not used
  INTCONbits.RBIF = 0;
  
  //Initialize variables  
  pll_rx_data_valid = 0;
  fm_valid_valid = 0;
  am_valid_valid = 0;
  
  for (i = 0; i < NUM_OF_BYTES_FOR_FM; i++) {
    fm_valid[i] = 0;
  }
  for (i = 0; i < NUM_OF_BYTES_FOR_AM; i++) {
    am_valid[i] = 0;
  }
  
  sent_pll_valid = 1;
  no_audio = 0;   
  power_sw_on = 0;
  
  tmr1_cnt = 0;
  
  while(1){ 
    //Re-activate TIMER3  
    TMR3H = 0;
    TMR3L = 0;
  
    T3GGO = 1;
    
    while (T3GGO == 1);//Wait for FIP H pulse ends
 
    fip_pulse_width = TMR3L;
    fip_pulse_width += (TMR3H << 8);

    if ((fip_pulse_width < FIP_PULSE_WIDTH_TH / 4) || (fip_pulse_width > FIP_PULSE_WIDTH_TH * 2)) {
      continue;
    }
    if(fip_pulse_width > FIP_PULSE_WIDTH_TH) { 
      power_sw_on = 1;
    }
    break;
  }

  //Re-activate TIMER3  
  TMR3H = 0;
  TMR3L = 0;
  
  T3GGO = 1;
  
  
//Enable interrupts
  PIR1 = 0x00;

  GIE_GIEH = 1;
  PEIE_GIEL = 1;

///End of device initialization  

  //Now start to control Pi
  if(power_sw_on == 1) { //Tuner power on?
    pi_state = power_on;      
  }
  else {
    pi_state = power_off;    
  }
//State machine loop
  while (1) {
    switch (pi_state) {
      case power_on:
             if (pi_power_on() == 0){ //Power on succeeded
               if(power_sw_on == 0) { //Tuner power off?
                 pi_state = shutdown;
               }
               else {
                 pi_state = operation;
               }
             }
             else {
               pi_state = force_shutdown; //For Pi Power on fail
             }
             break;
    
      case operation:
             if (pi_operation() == 0) {
               if(power_sw_on == 0) { //Tuner power off?
                 pi_state = shutdown;      
               }
             }
             else {
               pi_state = force_shutdown;
             }
             break;
    
      case shutdown:
             if(pi_shutdown() == 0) {
               pi_state = power_off;
             }
             else {
               pi_state = force_shutdown;
             }
             break;
    
      case force_shutdown:
             pi_force_shutdown();
             pi_state = power_off;
             break;

      case power_off:
             pi_power_off();
             if(power_sw_on == 1) { //Tuner power on?
               pi_state = power_on;      
             }
             break;
      
      default:
             pi_state = power_on; // Just in case
             
    }
    if (T3GGO == 0) { //check FIP H pulse ends?
      fip_pulse_width = TMR3L;
      fip_pulse_width += (TMR3H << 8);

      if ((fip_pulse_width >= FIP_PULSE_WIDTH_TH / 4) && (fip_pulse_width <= FIP_PULSE_WIDTH_TH * 2)) {
        
        if(fip_pulse_width > FIP_PULSE_WIDTH_TH) { 
          power_sw_on = 1;
        }
        else {
          power_sw_on = 0;
        }
      }
      //Re-activate TIMER3  
      TMR3H = 0;
      TMR3L = 0;

      T3GGO = 1;
    }
  }
}  

void pi_power_off(void) { 
  if (PORTCbits.RC2 == 1) { //if Pi 5V on
    PORTCbits.RC2 = 0; //Pi 5V off
    __delay_ms(POWER_OFF_TO_ON_INTERVAL);// To keep off state 
  }
}
 
char pi_power_on(void) {
  unsigned char i;
  char wk;
  char ret;
  
  PORTBbits.RB4 = 0; //shutdown inactive
  PORTCbits.RC2 = 1; //Pi 5V on
 

  ret = 1; //Pi not respond
  for (i = 0; i < PI_STATUS_MAX_WAIT_TIME_FOR_INIT / 500; disable_time_out ? i : i++) {  //maximum waiting time
    if (eusart1RxCount >= 1) {
      wk = EUSART1_Read();
      if ((wk == Pi2PIC_Pi_BOOTUP)) { //ready
        ret = 0; //Pi respond
        EUSART1_Write(PIC2Pi_REQ_VALID_TBL); //Allow to proceed
        break;
      }
    }  
    else {
      __delay_ms(500);//500ms
      continue;
    }
  }
 
  return ret;
 
}

 
  
//Pi shutdown  
char pi_shutdown(void) {
  int i;
  char ret;
  
  disable_audio_check();// Silence
  //Need to add timeout
  EUSART1_Write(PIC2Pi_SHUTDOWN);

  ret = 1;
  for(i = 0; i < SHUTDOWN_MAX_WAIT_TIME/500; disable_time_out ? i : i++) {
    __delay_ms(500);//0.5sec
  
    if(PORTBbits.RB1 == 1) {
      //In case of....
      __delay_ms(SHUTDOWN_COMP_TO_POWER_OFF_INTERVAL);
      ret = 0;
      break;
    }
  }

  return ret;  
}

//Pi force shutdown  
void pi_force_shutdown(void) {
  int i;

  disable_audio_check();// Silence
  PORTBbits.RB4 = 1;//Pi will detect this as shutdown request
  __delay_us(10);
  PORTBbits.RB4 = 0;

  for(i = 0; i < FORCE_SHUTDOWN_MAX_WAIT_TIME/500; disable_time_out ? i : i++) {
    __delay_ms(500);//0.5sec
    if(PORTBbits.RB1 == 1) {
      //In case of....
      __delay_ms(SHUTDOWN_COMP_TO_POWER_OFF_INTERVAL);
      break;
    }
  }
}
 
//Pi play radio or stop
char pi_operation(void) { 
  unsigned char wk;
  unsigned int  i, j;
  
  
  char ret;
  
  static LM7001_pll latest_pll = {fm_chn2pll_d(0), 1, 0};
  
  //Receive data from Pi
  if (eusart1RxCount != 0) { 
    wk = EUSART1_Read(); //command
    switch(wk) {
      case Pi2PIC_REGIST_FM_VALID:

						fm_valid_valid = receive_valid(fm_valid, NUM_OF_BYTES_FOR_FM);

            break;

	    case Pi2PIC_REGIST_AM_VALID:

						am_valid_valid = receive_valid(am_valid, NUM_OF_BYTES_FOR_AM);

            break;

      case Pi2PIC_REQUEST_FREQ:
            //Reply current frequency
            if (latest_pll.s == 1 && fm_valid_valid == 1) {
              EUSART1_Write(PIC2Pi_FM_PLL);
              EUSART1_Write(latest_pll.d & 0x7f);//MSB is always 0
              EUSART1_Write(latest_pll.d >> 7);
              sent_pll_valid = latest_pll.valid;

            }
            else if (latest_pll.s == 0 && am_valid_valid == 1) {
              EUSART1_Write(PIC2Pi_AM_PLL);
              EUSART1_Write(latest_pll.d & 0x7f);//MSB is always 0
              EUSART1_Write(latest_pll.d >> 7);

              sent_pll_valid = latest_pll.valid;
            }
              
            if (latest_pll.valid == 1) {
              enable_audio_check(); // Playing
            }
            else {
              disable_audio_check();// Silence
            }
            break;
              
      case Pi2PIC_Pi_BOOTUP:
            EUSART1_Write(PIC2Pi_REQ_VALID_TBL); //Allow to proceed
            break;
        
      default:
            ;
    } 
  }

  if (pll_rx_data_valid == 1) { 
    PIE1bits.SSP1IE = 0;//To avoid getting wrong pll_d, actually not needed
    INTCON3bits.INT2IE = 0;
    pll_rx_data_valid = 0;
    latest_pll.d = pll.d;
    latest_pll.s = pll.s;
    latest_pll.valid = pll.valid;
    INTCON3bits.INT2IE = 1;
    PIE1bits.SSP1IE = 1;

    if ((sent_pll_valid == 1) || (latest_pll.valid == 1)) { 
                
      if ((latest_pll.s == 1) && (fm_valid_valid == 1)) {
        //Play FM
        EUSART1_Write(PIC2Pi_FM_PLL);
        EUSART1_Write(latest_pll.d & 0x7f);//MSB must be always 0
        EUSART1_Write(latest_pll.d >> 7);
    
        sent_pll_valid = latest_pll.valid;
        
      }
      else if ((latest_pll.s == 0) && (am_valid_valid == 1)) {
        //Play AM
        EUSART1_Write(PIC2Pi_AM_PLL);
        EUSART1_Write(latest_pll.d & 0x7f);//MSB must beps -ef always 0
        EUSART1_Write(latest_pll.d >> 7);

        sent_pll_valid = latest_pll.valid;

      }
      if (latest_pll.valid == 1) {
        enable_audio_check(); // Playing
      }
      else {
        disable_audio_check();// Silence
      }
    }
  }
  
  ret = 0;
  if( no_audio == 1) {
    disable_audio_check();
    no_audio =0;

    EUSART1_Write(PIC2Pi_CHECK_STATUS);

    ret = 1; //Pi not respond
    for (i = 0; i < PI_STATUS_MAX_WAIT_TIME / 100; disable_time_out ? i : i++) {//maximum waiting time
      if (eusart1RxCount >= 1) {
        wk = EUSART1_Read();
        if ((wk == Pi2PIC_Pi_READY)) { //ready
          EUSART1_Write(PIC2Pi_RESTART_PLAYER);
        
          if (latest_pll.valid == 1) {
            enable_audio_check(); // Playing
            ret = 0; //Pi respond
          }  
        }
        break;
      }  
      else {
        __delay_ms(100);//100ms
        continue;
      }
    }
  }
  return ret;
}


void spi_pll_rx_handler() {
//It takes 11us with C and assembly code, 111us with only C code
  unsigned char  spi_data;
  unsigned int   wk;
  unsigned char  i;

  PIR1bits.SSP1IF= 0; //MCC generated interrupt manager does not have clearing SSP1IF
  
  spi_data = SPI1_ReadByte();
 
  if (pll_rx_en == 1) {
    if (spi_buf_wp < SPI_CMD_BYTES) {
      spi_buf[spi_buf_wp] = spi_data;
      spi_buf_wp++;
    }
    if (spi_buf_wp == SPI_CMD_BYTES) {
      
#ifndef USE_ASM      
      wk = (spi_buf[0] << 8) + spi_buf[1];
      //Invert bit order
      pll.d = 0;
      for (i = 0; i <14; i++) {//PLL data: d13 to d0
        if ((wk & (1 << (15 - i))) != 0) {
          pll.d |= (1 << i);
        };  
      }
#else
  #asm
      //Equivalent function with the above C code
      BANKSEL(_spi_buf)
      MOVF  BANKMASK(_spi_buf), W

      BANKSEL(_v_asm_h0)
      MOVWF BANKMASK(_v_asm_h0)

      BANKSEL(_spi_buf+1)
      MOVF  BANKMASK(_spi_buf+1), W

      BANKSEL(_v_asm_l0)
      MOVWF BANKMASK(_v_asm_l0)

      RLCF  _v_asm_h0   ;D7 - D0
      RRCF  _v_asm_l1
      RLCF  _v_asm_h0
      RRCF  _v_asm_l1
      RLCF  _v_asm_h0
      RRCF  _v_asm_l1
      RLCF  _v_asm_h0
      RRCF  _v_asm_l1
      RLCF  _v_asm_h0
      RRCF  _v_asm_l1
      RLCF  _v_asm_h0
      RRCF  _v_asm_l1
      RLCF  _v_asm_h0
      RRCF  _v_asm_l1
      RLCF  _v_asm_h0
      RRCF  _v_asm_l1

      CLRF  _v_asm_h1  ;D13 - D8
      RLCF  _v_asm_l0
      RRCF  _v_asm_h1
      RLCF  _v_asm_l0
      RRCF  _v_asm_h1
      RLCF  _v_asm_l0
      RRCF  _v_asm_h1
      RLCF  _v_asm_l0
      RRCF  _v_asm_h1
      RLCF  _v_asm_l0
      RRCF  _v_asm_h1
      RLCF  _v_asm_l0
      RRCF  _v_asm_h1
      RRNCF _v_asm_h1  ;D15, D14 are always 0
      RRNCF _v_asm_h1

  #endasm
      pll.d = (v_asm_h1 << 8) + v_asm_l1;
#endif
      pll.s = spi_buf[2] & 0x01;      
      if ( pll.s == 0x01) {// FM
        //FM
        wk = pll_d2fm_chn(pll.d);
        if (wk < NUM_OF_FM_CHN) { 
          
          pll_rx_data_valid = 1; 
          
          if ((fm_valid[wk / 8] & (1 << (wk % 8))) != 0) {

            PORTAbits.RA4 = 1;//Tuned indicator on
            asm("NOP");       //wait cycle to ensure consecutive bit access
            PORTAbits.RA3 = 1;//Stereo indicator on

            pll.valid = 1;
          }
          else {

            PORTAbits.RA4 = 0;//Tuned indicator off
            asm("NOP");       //wait cycle to ensure consecutive bit access
            PORTAbits.RA3 = 0;//Stereo indicator off

            pll.valid = 0;
          }
        }
      }
      else {
        wk = pll_d2am_chn(pll.d);
        if (wk < NUM_OF_AM_CHN) {
         
          pll_rx_data_valid = 1;  

          if ((am_valid[wk / 8] & (1 << (wk % 8))) != 0) {

            PORTAbits.RA4 = 1;//Tuned indicator on
            asm("NOP");       //wait cycle to ensure consecutive bit access
            PORTAbits.RA3 = 1;//Stereo indicator on

            pll.valid = 1;
          }
          else {

            PORTAbits.RA4 = 0;//Tuned indicator off
            asm("NOP");       //wait cycle to ensure consecutive bit access
            PORTAbits.RA3 = 0;//Stereo indicator off

            pll.valid = 0;
          }
        }
      
      }              
      pll_rx_en = 0;
//      pll_rx_data_valid = 1;  
    }
  }
}

void int2_pll_rx_en_handler() {

  spi_buf_wp = 0;
  pll_rx_data_valid = 0;  
  pll_rx_en = 1;
}

void timer1_int_handler() {
  
  if (INTCONbits.RBIF == 0) {
    if (tmr1_cnt < ( NO_AUDIO_MAX_TIME - 1)) {
      disable_time_out ? tmr1_cnt : tmr1_cnt++;
    }
    else {
      no_audio = 1;

    }
  }
  else {
    PORTBbits.RB5; //Read to set the pin state IOC latch
    asm("NOP"); //to put wait cycle, see interrupt-on-change section on Data Sheet
    INTCONbits.RBIF = 0;
    tmr1_cnt = 0;
  }
}

void enable_audio_check() {

  PIE1bits.TMR1IE = 0; //Disable TMR1 interrupt
  
  PORTBbits.RB5; //Read to set the pin state IOC latch
  asm("NOP"); //to put wait cycle, see interrupt-on-change section on Data Sheet
  INTCONbits.RBIF = 0;

  no_audio = 0;
  tmr1_cnt = 0;
  
  TMR1_Reload();
  TMR1_StartTimer();
  PIE1bits.TMR1IE = 1; //Enable TMR1 interrupt
}

void disable_audio_check() {

  PIE1bits.TMR1IE = 0; //Disable TMR1 interrupt
  TMR1_StopTimer();
  no_audio = 0;
}



unsigned char receive_valid(char *valid_p, int num_of_bytes) {
  unsigned char wk;
  unsigned int  i, j;
  unsigned int  chksum;
  unsigned int  chksum_gold;
  
  //buffer for 8-7encoded valid including chksum
  unsigned char valid7;

  char ret;
  

	//receive xm_valid with 7-8 decoding
	for (i = j = 0; i < bytes_8_7encoded(num_of_bytes); i++) {
		valid7 = EUSART1_Read();
		if ((i % 8) == 0) { 
			wk = valid7; //MSB data for each 7 bytes
		}
		else {
			if((wk & 0x01) == 0x01) {
				valid_p[j] = valid7 | 0x80;
			}
			else {
				valid_p[j] = valid7;
			}

			wk = wk >> 1; 
			j++;
		}
	}

	//Check chksum
	chksum = 0;
	for (i = 0; i < num_of_bytes - 2; i++) {
		chksum += valid_p[i];
	}

	chksum_gold = (valid_p[i] << 8) + valid_p[i + 1];
	valid_p[i] = 0;     //Clear received chksum data
	valid_p[i + 1] = 0;

	if (chksum == chksum_gold) {
		ret = 1;
	}
	else {
		for (i = 0; i < num_of_bytes; i++) {
			valid_p[i] = 0;
		}
		ret = 0;
	}

	return ret;
}


/**
 End of File
*/
